# 1 "g10.c"
 



















 





# 1 "../config.h" 1
 
 
 





















 





 
 

 
 

 

 

 


 


 
 

 


 


 
 

 
 

 
 

 
 

 






 

 


 






 


 


 
 

 
 



 


 


 
 

 


 



 

 


 


 


 


 
 

 


 
 

 


 


 


 


 


 


 


 


 


 


 


 


 


 


 


 
 

 


 


 


 


 


 


 


 


 


 


 


 


 
 


 
















# 27 "g10.c" 2

# 1 "/usr/include/errno.h" 1 3
 

















 






# 1 "/usr/include/features.h" 1 3
 





















 




































 









 







 



 





 








 



































 
# 1 "/usr/include/sys/cdefs.h" 1 3
 






















# 1 "/usr/include/features.h" 1 3
 

















# 136 "/usr/include/features.h" 3

# 24 "/usr/include/sys/cdefs.h" 2 3










 


 


# 50 "/usr/include/sys/cdefs.h" 3

   


















# 100 "/usr/include/sys/cdefs.h" 3








 







 

typedef long double __long_double_t;

# 128 "/usr/include/sys/cdefs.h" 3


 




 







 













# 134 "/usr/include/features.h" 2 3



# 26 "/usr/include/errno.h" 2 3

# 1 "/usr/include/linux/errno.h" 1 3



# 1 "/usr/include/asm/errno.h" 1 3

































































































































# 4 "/usr/include/linux/errno.h" 2 3


# 14 "/usr/include/linux/errno.h" 3



# 27 "/usr/include/errno.h" 2 3



extern int sys_nerr;
extern char *sys_errlist[];


extern int _sys_nerr;
extern char *_sys_errlist[];


 

extern void	perror  (__const char* __s)  ;
extern char*	strerror  (int __errno)  ;





extern int errno;






 


# 28 "g10.c" 2

# 1 "/usr/include/stdio.h" 1 3
 



















 












# 1 "/usr/include/libio.h" 1 3
 























 




# 1 "/usr/include/_G_config.h" 1 3
  




















typedef long _G_clock_t;
typedef unsigned short _G_dev_t;
typedef long int _G_fpos_t;
typedef unsigned short _G_gid_t;
typedef unsigned long _G_ino_t;
typedef unsigned short _G_mode_t;
typedef unsigned short _G_nlink_t;
typedef long _G_off_t;
typedef int _G_pid_t;



typedef int _G_ptrdiff_t;
typedef unsigned long _G_sigset_t;



typedef unsigned int _G_size_t;
typedef long _G_time_t;
typedef unsigned short _G_uid_t;
typedef long int _G_wchar_t;
typedef int _G_ssize_t;
typedef int   _G_wint_t;
typedef void * _G_va_list;




typedef          int   _G_int8_t __attribute__((__mode__(__QI__)));
typedef unsigned int  _G_uint8_t __attribute__((__mode__(__QI__)));
typedef          int  _G_int16_t __attribute__((__mode__(__HI__)));
typedef unsigned int _G_uint16_t __attribute__((__mode__(__HI__)));
typedef          int  _G_int32_t __attribute__((__mode__(__SI__)));
typedef unsigned int _G_uint32_t __attribute__((__mode__(__SI__)));
typedef          int  _G_int64_t __attribute__((__mode__(__DI__)));
typedef unsigned int _G_uint64_t __attribute__((__mode__(__DI__)));








































# 30 "/usr/include/libio.h" 2 3













# 51 "/usr/include/libio.h" 3


# 63 "/usr/include/libio.h" 3


 







































 























 

















 


struct _IO_lock_t {
  void *ptr;
  short int field1;
  short int field2;
};

 

struct _IO_jump_t;  struct _IO_FILE;

struct _IO_marker {
  struct _IO_marker *_next;
  struct _IO_FILE *_sbuf;
   

   
  int _pos;
# 176 "/usr/include/libio.h" 3

};



struct _IO_FILE {








  int _flags;		 
   
   
  char* _IO_read_ptr;	 
  char* _IO_read_end;	 
  char* _IO_read_base;	 

  char* _IO_write_base;	 
  char* _IO_write_ptr;	 
  char* _IO_write_end;	 
  char* _IO_buf_base;	 
  char* _IO_buf_end;	 
   
  char *_IO_save_base;  
  char *_IO_backup_base;   
  char *_IO_save_end;  

  struct _IO_marker *_markers;
  
  struct _IO_FILE *_chain;
  
  int _fileno;
  int _blksize;
  _G_off_t  _offset;
  

   
  unsigned short _cur_column;
  char _unused;
  char _shortbuf[1];
  
   

  struct _IO_lock_t _IO_lock;
};


typedef struct _IO_FILE _IO_FILE;


struct _IO_FILE_plus;
extern struct _IO_FILE_plus _IO_stdin_, _IO_stdout_, _IO_stderr_;








extern int __underflow  (_IO_FILE*)  ;
extern int __uflow  (_IO_FILE*)  ;
extern int __overflow  (_IO_FILE*, int)  ;

















 



extern int _IO_vfscanf  (_IO_FILE*, const char*, _G_va_list , int*)  ;
extern int _IO_vfprintf  (_IO_FILE*, const char*, _G_va_list )  ;
extern _G_ssize_t  _IO_padn  (_IO_FILE *, int, _G_ssize_t )  ;
extern _G_size_t  _IO_sgetn  (_IO_FILE *, void*, _G_size_t )  ;

extern _G_fpos_t  _IO_seekoff  (_IO_FILE*, _G_off_t , int, int)  ;
extern _G_fpos_t  _IO_seekpos  (_IO_FILE*, _G_fpos_t , int)  ;

extern void _IO_free_backup_area  (_IO_FILE*)  ;






# 34 "/usr/include/stdio.h" 2 3

























  











typedef _G_size_t  size_t;




typedef struct _IO_FILE FILE;
typedef _G_fpos_t  fpos_t;




 






 

extern FILE *stdin, *stdout, *stderr;

# 102 "/usr/include/stdio.h" 3








 

extern void clearerr  (FILE*)  ;
extern int fclose  (FILE*)  ;
extern int feof  (FILE*)  ;
extern int ferror  (FILE*)  ;
extern int fflush  (FILE*)  ;
extern int fgetc  (FILE *)  ;
extern int fgetpos  (FILE* fp, fpos_t *pos)  ;
extern char* fgets  (char*, int, FILE*)  ;
extern FILE* fopen  (__const char*, __const char*)  ;
extern int fprintf  (FILE*, __const char* format, ...)  ;
extern int fputc  (int, FILE*)  ;
extern int fputs  (__const char *str, FILE *fp)  ;
extern size_t fread  (void*, size_t, size_t, FILE*)  ;
extern FILE* freopen  (__const char*, __const char*, FILE*)  ;
extern int fscanf  (FILE *fp, __const char* format, ...)  ;
extern int fseek  (FILE* fp, long int offset, int whence)  ;
extern int fsetpos  (FILE* fp, __const fpos_t *pos)  ;
extern long int ftell  (FILE* fp)  ;
extern size_t fwrite  (__const void*, size_t, size_t, FILE*)  ;
extern int getc  (FILE *)  ;
extern int getchar  (void)  ;
extern char* gets  (char*)  ;
extern void perror  (__const char *)  ;
extern int printf  (__const char* format, ...)  ;
extern int putc  (int, FILE *)  ;
extern int putchar  (int)  ;
extern int puts  (__const char *str)  ;
extern int remove  (__const char*)  ;
extern int rename  (__const char* _old, __const char* _new)  ;
extern void rewind  (FILE*)  ;
extern int scanf  (__const char* format, ...)  ;
extern void setbuf  (FILE*, char*)  ;
extern void setlinebuf  (FILE*)  ;
extern void setbuffer  (FILE*, char*, int)  ;
extern int setvbuf  (FILE*, char*, int mode, size_t size)  ;
extern int sprintf  (char*, __const char* format, ...)  ;
extern int sscanf  (__const char* string, __const char* format, ...)  ;
extern FILE* tmpfile  (void)  ;
extern char* tmpnam  (char*)  ;
extern int ungetc  (int c, FILE* fp)  ;
extern int vfprintf  (FILE *fp, char __const *fmt0, _G_va_list)  ;
extern int vprintf  (char __const *fmt, _G_va_list)  ;
extern int vsprintf  (char* string, __const char* format, _G_va_list)  ;


extern int vfscanf  (FILE*, __const char *, _G_va_list)  ;
extern int vscanf  (__const char *, _G_va_list)  ;
extern int vsscanf  (__const char *, __const char *, _G_va_list)  ;

extern int getw  (FILE*)  ;
extern int putw  (int, FILE*)  ;

extern char* tempnam  (__const char *__dir, __const char *__pfx)  ;





extern int sys_nerr;
extern char *sys_errlist[];


extern int _sys_nerr;
extern char *_sys_errlist[];

 

 
extern void psignal  (int __sig, __const char *__s)  ;







extern _G_ssize_t  getdelim  (char **, size_t *, int, FILE*)  ;





extern int snprintf  (char *, size_t, const char *, ...)  ;
extern int vsnprintf  (char *, size_t, const char *, _G_va_list)  ;

extern int asprintf  (char **, const char *, ...)  ;
extern int vasprintf  (char **, const char *, _G_va_list)  ;



extern FILE *fdopen  (int, __const char *)  ;
extern int fileno  (FILE*)  ;
extern FILE* popen  (__const char*, __const char*)  ;
extern int pclose  (FILE*)  ;




 
extern char *ctermid  (char *__buf)  ;

 
extern char *cuserid  (char * __buf)  ;



extern int __underflow  (struct _IO_FILE*)  ;
extern int __overflow  (struct _IO_FILE*, int)  ;

# 234 "/usr/include/stdio.h" 3











 


# 29 "g10.c" 2

# 1 "/usr/include/stdlib.h" 1 3
 

















 



 






 



# 1 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 1 3





 


# 17 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 3



 


 





 


# 59 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 3


 





 


















 





 

 

# 129 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 3


 

 


# 180 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 3





 




 



























 






















typedef long int wchar_t;
















 

# 288 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 3




 



















# 34 "/usr/include/stdlib.h" 2 3


 







 

 
typedef struct
  {
    int quot;			 
    int rem;			 
  } div_t;

 
typedef struct
  {
    long int quot;		 
    long int rem;		 
  } ldiv_t;


 





 





 




 
extern double atof  (__const char *__nptr)  ;
 
extern int atoi  (__const char *__nptr)  ;
 
extern long int atol  (__const char *__nptr)  ;

 
extern long long int atoq  (__const char *__nptr)  ;


 
extern float strtof  (__const char *__nptr, char **__endptr)  ;
 
extern double strtod  (__const char *__nptr, char **__endptr)  ;

 
extern __long_double_t strtold  (__const char *__nptr, char **__endptr)  ;

 
extern long int strtol  (__const char *__nptr, char **__endptr,
			     int __base)  ;
 
extern unsigned long int strtoul  (__const char *__nptr,
				       char **__endptr, int __base)  ;

 
extern long long int strtoq  (__const char *__nptr, char **__endptr,
				int __base)  ;
 
extern unsigned long long int strtouq  (__const char *__nptr,
					char **__endptr, int __base)  ;


 


extern double __strtod_internal (__const char *__nptr,
				 char **__endptr, int __group);
extern float __strtof_internal (__const char *__nptr, char **__endptr,
				int __group);
extern __long_double_t __strtold_internal (__const char *__nptr,
					   char **__endptr, int __group);
extern long int __strtol_internal (__const char *__nptr, char **__endptr,
				   int __base, int __group);
extern unsigned long int __strtoul_internal (__const char *__nptr,
					     char **__endptr, int __base,
					     int __group);

extern long long int __strtoq_internal (__const char *__nptr, char **__endptr,
					int __base, int __group);
extern unsigned long long int __strtouq_internal (__const char *__nptr,
						  char **__endptr, int __base,
						  int __group);


# 170 "/usr/include/stdlib.h" 3



 
extern int rand  (void)  ;
 
extern void srand  (unsigned int __seed)  ;

 



 
extern long int __random  (void)  ;
 
extern void __srandom  (unsigned int __seed)  ;

 



extern void *  __initstate  (unsigned int __seed, void *  __statebuf,
				 size_t __statelen)  ;
 

extern void *  __setstate  (void *  __statebuf)  ;


extern long int random  (void)  ;
extern void srandom  (unsigned int __seed)  ;
extern void *  initstate  (unsigned int __seed, void *  __statebuf,
			       size_t __statelen)  ;
extern void *  setstate  (void *  __statebuf)  ;

# 214 "/usr/include/stdlib.h" 3




 
extern void *  malloc  (size_t __size)  ;
 

extern void *  realloc  (void *  __ptr, size_t __size)  ;
 
extern void *  calloc  (size_t __nmemb, size_t __size)  ;
 
extern void free  (void *  __ptr)  ;


 
extern void cfree  (void *  __ptr)  ;



# 1 "/usr/include/alloca.h" 1 3
 






















# 1 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 1 3





 


# 17 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 3



 


 





 


# 59 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 3


 





 


















 





 

 

# 129 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 3


 

 


# 180 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 3





 




 

# 256 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 3


 

# 288 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 3




 



















# 24 "/usr/include/alloca.h" 2 3


 

 



 
extern void *  __alloca  (size_t __size)  ;

 










 


# 234 "/usr/include/stdlib.h" 2 3




 
extern void *  valloc  (size_t __size)  ;



 
extern   void abort  (void)  ;


 
extern int atexit  (void (*__func) (void))  ;


 

extern int on_exit  (void (*__func) (int __status, void *  __arg),
			 void *  __arg)  ;


 


extern   void exit  (int __status)  ;


 
extern char *getenv  (__const char *__name)  ;


 
 

extern int putenv  (__const char *__string)  ;



 

extern int setenv  (__const char *__name, __const char *__value,
			int __replace)  ;


 
extern int system  (__const char *__command)  ;




 
typedef int (*__compar_fn_t)  (__const void * , __const void * )  ;



typedef __compar_fn_t comparison_fn_t;


 

extern void *  bsearch  (__const void *  __key, __const void *  __base,
			     size_t __nmemb, size_t __size,
			     __compar_fn_t __compar)  ;

 

extern void qsort  (void *  __base, size_t __nmemb, size_t __size,
			__compar_fn_t __compar)  ;


# 314 "/usr/include/stdlib.h" 3


 
extern   int abs  (int __x)  ;
extern   long int labs  (long int __x)  ;


 

 
extern   div_t div  (int __numer, int __denom)  ;
extern   ldiv_t ldiv  (long int __numer, long int __denom)  ;


 

extern int mblen  (__const char *__s, size_t __n)  ;
 

extern int mbtowc  (wchar_t * __pwc, __const char *__s, size_t __n)  ;
 

extern int wctomb  (char *__s, wchar_t __wchar)  ;







 
extern size_t mbstowcs  (wchar_t * __pwcs, __const char *__s, size_t __n)  ;
 
extern size_t wcstombs  (char *__s, __const wchar_t * __pwcs, size_t __n)  ;

 




 





 
# 375 "/usr/include/stdlib.h" 3


 


extern char **environ;
extern char **__environ;

extern char*	ecvt  (double __value, size_t __ndigit, int *__decpt,
			int *__sign)  ;
extern char*	fcvt  (double __value, size_t __ndigit, int *__decpt,
			int *__sign)  ;
extern char*	gcvt  (double __value, size_t __ndigit, char *__buf)  ;

extern double	drand48  (void)  ;
extern double	erand48  (unsigned short int __xsubi[3])  ;
extern long int	lrand48  (void)  ;
extern long int	nrand48  (unsigned short int __xsubi[3])  ;
extern long int	mrand48  (void)  ;
extern long int	jrand48  (unsigned short int __xsubi[3])  ;
extern void	srand48  (long int __seedval)  ;
extern unsigned short int
			*seed48  (unsigned short int __seed16v[3])  ;
extern void	lcong48  (unsigned short int __param[7])  ;

extern int	setenv  (__const char *__name, __const char *__value,
			int __overwrite)  ;
extern void	unsetenv  (__const char *__name)  ;

struct qelem {
  struct qelem *q_forw;
  struct qelem *q_back;
  char q_data[1];
};

extern void insque  (struct qelem *__elem, struct qelem *__prev)  ;
extern void remque  (struct qelem *__elem)  ;



 


# 30 "g10.c" 2

# 1 "/usr/include/string.h" 1 3
 

















 








 

 


# 1 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 1 3





 


# 17 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 3



 


 





 


# 59 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 3


 





 


















 





 

 

# 129 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 3


 

 


# 180 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 3





 




 

# 256 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 3


 

# 288 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 3




 



















# 33 "/usr/include/string.h" 2 3


 






 

extern void *  memmove  (void *  __dest, __const void *  __src,
			     size_t __n)  ;

 


extern void *  __memccpy  (void *  __dest, __const void *  __src,
			       int __c, size_t __n)  ;

extern void *  memccpy  (void *  __dest, __const void *  __src,
			     int __c, size_t __n)  ;



 
extern void *  memset  (void *  __s, int __c, size_t __n)  ;

 






 
extern void *  memchr  (__const void *  __s, int __c, size_t __n)  ;


 
extern char *strcpy  (char *__dest, __const char *__src)  ;
 
extern char *strncpy  (char *__dest, __const char *__src, size_t __n)  ;

 
extern char *strcat  (char *__dest, __const char *__src)  ;
 
extern char *strncat  (char *__dest, __const char *__src, size_t __n)  ;

 
extern int strcmp  (__const char *__s1, __const char *__s2)  ;
 
extern int strncmp  (__const char *__s1, __const char *__s2, size_t __n)  ;

 
extern int strcoll  (__const char *__s1, __const char *__s2)  ;
 
extern size_t strxfrm  (char *__dest, __const char *__src, size_t __n)  ;


 
extern char *strdup  (__const char *__s)  ;


 
extern char *strchr  (__const char *__s, int __c)  ;
 
extern char *strrchr  (__const char *__s, int __c)  ;

 

extern size_t strcspn  (__const char *__s, __const char *__reject)  ;
 

extern size_t strspn  (__const char *__s, __const char *__accept)  ;
 
extern char *strpbrk  (__const char *__s, __const char *__accept)  ;
 
extern char *strstr  (__const char *__haystack, __const char *__needle)  ;
 
extern char *strtok  (char *__s, __const char *__delim)  ;


 


extern void *  memmem  (__const void *  __haystack,
			    size_t __haystacklen,
			    __const void *  __needle,
			    size_t __needlelen)  ;


 
extern size_t strlen  (__const char *__s)  ;

 
extern char *strerror  (int __errnum)  ;


 
extern char *index  (__const char *__s, int __c)  ;

 
extern char *rindex  (__const char *__s, int __c)  ;

# 149 "/usr/include/string.h" 3


 
extern void bcopy  (__const void *  __src, void *  __dest, int __n)  ;

 
extern void bzero  (void *  __s, int __n)  ;

 
extern int bcmp  (__const void *  __s1, __const void *  __s2, int __n)  ;



 

extern int ffs  (int __i)  ;

 
extern int strcasecmp  (__const char *__s1, __const char *__s2)  ;

 

extern char *strsep  (char **__stringp, __const char *__delim)  ;



 
extern int strncasecmp  (__const char *__s1, __const char *__s2,
			     size_t __n)  ;

 
extern char *strsignal  (int __sig)  ;

 
extern char *stpcpy  (char *__dest, __const char *__src)  ;

 

extern char *__stpncpy  (char *__dest, __const char *__src, size_t __n)  ;
extern char *stpncpy  (char *__dest, __const char *__src, size_t __n)  ;

 
extern char *strfry  (char *__string)  ;

 
extern void *  memfrob  (void *  __s, size_t __n)  ;

extern void swab  (__const void *  __from, void *  __to,
                        size_t __nbytes)  ;


 


# 31 "g10.c" 2

# 1 "/usr/include/unistd.h" 1 3
 

















 








 

 


 


 




 



 



 



 




 


























# 1 "/usr/include/posix_opt.h" 1 3



 


 


 




 



























 









# 85 "/usr/include/unistd.h" 2 3



 





 

# 1 "/usr/include/gnu/types.h" 1 3
 






















 
typedef unsigned char __u_char;
typedef unsigned short __u_short;
typedef unsigned int __u_int;
typedef unsigned long __u_long;
typedef struct
{
  long val[2];
} __quad;
typedef struct
{
  __u_long val[2];
} __u_quad;


typedef unsigned short __dev_t;	 
typedef unsigned short __gid_t;	 
typedef unsigned short __uid_t;	 
typedef unsigned short __mode_t; 
typedef long	__daddr_t;	 
typedef long	__off_t;	 

typedef long long __loff_t;	 

typedef unsigned long __ino_t;	 
typedef unsigned short __nlink_t;	 
typedef long	__time_t;
# 61 "/usr/include/gnu/types.h" 3


typedef int __pid_t;		 
typedef int __ssize_t;		 
typedef __quad __fsid_t;	 

 
typedef char *__caddr_t;
typedef long int __swblk_t;	 


 





# 1 "/usr/include/linux/posix_types.h" 1 3



 









 










 


















typedef struct fd_set {
	unsigned long fds_bits [(1024 / (8 * sizeof(unsigned long)) ) ];
} __kernel_fd_set;

# 1 "/usr/include/asm/posix_types.h" 1 3



 





typedef unsigned short	__kernel_dev_t;
typedef unsigned long	__kernel_ino_t;
typedef unsigned short	__kernel_mode_t;
typedef unsigned short	__kernel_nlink_t;
typedef long		__kernel_off_t;
typedef int		__kernel_pid_t;
typedef unsigned short	__kernel_uid_t;
typedef unsigned short	__kernel_gid_t;
typedef unsigned int	__kernel_size_t;
typedef int		__kernel_ssize_t;
typedef int		__kernel_ptrdiff_t;
typedef long		__kernel_time_t;
typedef long		__kernel_clock_t;
typedef int		__kernel_daddr_t;
typedef char *		__kernel_caddr_t;


typedef long long	__kernel_loff_t;


typedef struct {



	int	__val[2];

} __kernel_fsid_t;



























# 48 "/usr/include/linux/posix_types.h" 2 3



# 78 "/usr/include/gnu/types.h" 2 3






typedef struct __fd_set {
        unsigned long fds_bits [(1024 / (8 * sizeof(unsigned long)) ) ];
} __fd_set;

# 128 "/usr/include/gnu/types.h" 3


# 152 "/usr/include/gnu/types.h" 3



# 96 "/usr/include/unistd.h" 2 3








# 1 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 1 3





 


# 17 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 3



 


 





 


# 59 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 3


 





 


















 





 

 

# 129 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 3


 

 


# 180 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 3





 




 

# 256 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 3


 

# 288 "/usr/lib/gcc-lib/i486-linux/2.7.2.1/include/stddef.h" 3




 



















# 104 "/usr/include/unistd.h" 2 3



 






 
extern int __access  (__const char *__name, int __type)  ;
extern int access  (__const char *__name, int __type)  ;


 






 




extern __off_t __lseek  (int __fd, __off_t __offset, int __whence)  ;
extern __off_t lseek  (int __fd, __off_t __offset, int __whence)  ;


 
extern __loff_t __llseek  (int __fd, __loff_t __offset, int __whence)  ;
extern __loff_t llseek  (int __fd, __loff_t __offset, int __whence)  ;


 
extern int __close  (int __fd)  ;
extern int close  (int __fd)  ;

 

extern __ssize_t  __read  (int __fd, void *  __buf, size_t __nbytes)  ;
extern __ssize_t  read  (int __fd, void *  __buf, size_t __nbytes)  ;

 
extern __ssize_t  __write  (int __fd, __const void *  __buf, size_t __n)  ;
extern __ssize_t  write  (int __fd, __const void *  __buf, size_t __n)  ;


 



extern int __pipe  (int __pipedes[2])  ;
extern int pipe  (int __pipedes[2])  ;

 






extern unsigned int alarm  (unsigned int __seconds)  ;

 






extern unsigned int sleep  (unsigned int __seconds)  ;


 

extern int pause  (void)  ;


 
extern int __chown  (__const char *__file,
			 __uid_t __owner, __gid_t __group)  ;
extern int chown  (__const char *__file,
		       __uid_t __owner, __gid_t __group)  ;


 
extern int __fchown  (int __fd,
			  __uid_t __owner, __gid_t __group)  ;
extern int fchown  (int __fd,
			__uid_t __owner, __gid_t __group)  ;
 

extern int __fchdir  (int __fd)  ;
extern int fchdir  (int __fd)  ;


 
extern int __chdir  (__const char *__path)  ;
extern int chdir  (__const char *__path)  ;

 






extern char *getcwd  (char *__buf, size_t __size)  ;


 


extern char *get_current_dir_name  (void)  ;



 


extern char *getwd  (char __buf [])  ;



 
extern int __dup  (int __fd)  ;
extern int dup  (int __fd)  ;

 
extern int __dup2  (int __fd, int __fd2)  ;
extern int dup2  (int __fd, int __fd2)  ;

 
extern char **__environ;
extern char **environ;


 

extern int __execve  (__const char *__path, char *__const __argv[],
			  char *__const __envp[])  ;
extern int execve  (__const char *__path, char *__const __argv[],
			char *__const __envp[])  ;

 
extern int execv  (__const char *__path, char *__const __argv[])  ;

 

extern int execle  (__const char *__path, __const char *__arg,...)  ;

 

extern int execl  (__const char *__path, __const char *__arg,...)  ;

 

extern int execvp  (__const char *__file, char *__const __argv[])  ;

 


extern int execlp  (__const char *__file, __const char *arg,...)  ;


 
extern   void _exit  (int __status)   __attribute__ ((noreturn)) ;


 



# 1 "/usr/include/confname.h" 1 3
 





















 
enum
  {
    _PC_LINK_MAX



    ,
    _PC_MAX_CANON,
    _PC_MAX_INPUT,
    _PC_NAME_MAX,
    _PC_PATH_MAX,
    _PC_PIPE_BUF,

    _PC_CHOWN_RESTRICTED,

    _PC_NO_TRUNC,
    _PC_VDISABLE,




     
    _PC_ASYNC_IO,
    _PC_PRIO_IO,
    _PC_SYNC_IO
  };

 
enum
  {
    _SC_ARG_MAX



    ,
    _SC_CHILD_MAX,
    _SC_CLK_TCK,
    _SC_NGROUPS_MAX,
    _SC_OPEN_MAX,


    _SC_STREAM_MAX,
    _SC_TZNAME_MAX,

    _SC_JOB_CONTROL,
    _SC_SAVED_IDS,
    _SC_VERSION,




    _SC_PAGESIZE,





     

    _SC_BC_BASE_MAX,
    _SC_BC_DIM_MAX,
    _SC_BC_SCALE_MAX,
    _SC_BC_STRING_MAX,
    _SC_COLL_WEIGHTS_MAX,
    _SC_EQUIV_CLASS_MAX,
    _SC_EXPR_NEST_MAX,
    _SC_LINE_MAX,
    _SC_RE_DUP_MAX,

    _SC_2_VERSION,
    _SC_2_C_BIND,
    _SC_2_C_DEV,
    _SC_2_FORT_DEV,
    _SC_2_FORT_RUN,
    _SC_2_SW_DEV,
    _SC_2_LOCALEDEF,

     
    _SC_AIO_LISTIO_MAX,
    _SC_AIO_MAX,
    _SC_AIO_PRIO_DELTA_MAX,
    _SC_DELAYTIMER_MAX,
    _SC_MQ_OPEN_MAX,
    _SC_MQ_PRIO_MAX,
    _SC_RTSIG_MAX,
    _SC_SEM_NSEMS_MAX,
    _SC_SEM_VALUE_MAX,
    _SC_SIGQUEUE_MAX,
    _SC_TIMER_MAX,
    _SC_ASYNCHRONOUS_IO,
    _SC_FSYNC,
    _SC_MAPPED_FILES,
    _SC_MEMLOCK,
    _SC_MEMLOCK_RANGE,
    _SC_MEMORY_PROTECTION,
    _SC_MESSAGE_PASSING,
    _SC_PRIORITIZED_IO,
    _SC_PRIORITY_SCHEDULING,
    _SC_REALTIME_SIGNALS,
    _SC_SEMAPHORES,
    _SC_SHARED_MEMORY_OBJECTS,
    _SC_SYNCHRONIZED_IO,
    _SC_TIMERS
  };


 
enum
  {
    _CS_PATH			 
  };


 





# 280 "/usr/include/unistd.h" 2 3


 
extern long int __pathconf  (__const char *__path, int __name)  ;
extern long int pathconf  (__const char *__path, int __name)  ;

 
extern long int __fpathconf  (int __fd, int __name)  ;
extern long int fpathconf  (int __fd, int __name)  ;


 
extern long int __sysconf  (int __name)  ;
extern long int sysconf  (int __name)  ;


 
extern size_t confstr  (int __name, char *__buf, size_t __len)  ;



 
extern __pid_t __getpid  (void)  ;
extern __pid_t getpid  (void)  ;

 
extern __pid_t __getppid  (void)  ;
extern __pid_t getppid  (void)  ;



 


extern __pid_t getpgid  (__pid_t __pid)  ;

 
extern __pid_t getpgrp  (void)  ;

 


extern int setpgid  (__pid_t __pid, __pid_t __pgid)  ;

extern int setpgrp  (void)  ;

# 350 "/usr/include/unistd.h" 3


 


extern __pid_t __setsid  (void)  ;
extern __pid_t setsid  (void)  ;

extern __pid_t __getsid  (__pid_t __pid)  ;
extern __pid_t getsid  (__pid_t __pid)  ;

 
extern __uid_t __getuid  (void)  ;
extern __uid_t getuid  (void)  ;

 
extern __uid_t __geteuid  (void)  ;
extern __uid_t geteuid  (void)  ;

 
extern __gid_t __getgid  (void)  ;
extern __gid_t getgid  (void)  ;

 
extern __gid_t __getegid  (void)  ;
extern __gid_t getegid  (void)  ;

 


extern int __getgroups  (int __size, __gid_t __list[])  ;
extern int getgroups  (int __size, __gid_t __list[])  ;

 



extern int __setuid  (__uid_t __uid)  ;
extern int setuid  (__uid_t __uid)  ;


 

extern int __setreuid  (__uid_t __ruid, __uid_t __euid)  ;
extern int setreuid  (__uid_t __ruid, __uid_t __euid)  ;

 
extern int seteuid  (__uid_t __uid)  ;


 



extern int __setgid  (__gid_t __gid)  ;
extern int setgid  (__gid_t __gid)  ;


 

extern int __setregid  (__gid_t __rgid, __gid_t __egid)  ;
extern int setregid  (__gid_t __rgid, __gid_t __egid)  ;

 
extern int setegid  (__uid_t __uid)  ;


 
extern int __setfsuid  (__uid_t __uid)  ;
extern int setfsuid  (__uid_t __uid)  ;

 
extern int __setfsgid  (__gid_t __gid)  ;
extern int setfsgid  (__gid_t __gid)  ;


 


extern __pid_t __fork  (void)  ;
extern __pid_t fork  (void)  ;


 



extern __pid_t __vfork  (void)  ;
extern __pid_t vfork  (void)  ;



# 450 "/usr/include/unistd.h" 3


 

extern char *ttyname  (int __fd)  ;

 

extern int __isatty  (int __fd)  ;
extern int isatty  (int __fd)  ;


 
extern int __link  (__const char *__from, __const char *__to)  ;
extern int link  (__const char *__from, __const char *__to)  ;


 
extern int __symlink  (__const char *__from, __const char *__to)  ;
extern int symlink  (__const char *__from, __const char *__to)  ;

 


extern int __readlink  (__const char *__path, char *__buf, size_t __len)  ;
extern int readlink  (__const char *__path, char *__buf, size_t __len)  ;



 
extern int __unlink  (__const char *__name)  ;
extern int unlink  (__const char *__name)  ;

 
extern int __rmdir  (__const char *__path)  ;
extern int rmdir  (__const char *__path)  ;


 
extern __pid_t tcgetpgrp  (int __fd)  ;

 
extern int tcsetpgrp  (int __fd, __pid_t __pgrp_id)  ;


 
extern char *getlogin  (void)  ;


 
extern int setlogin  (__const char *__name)  ;



 
























extern int getopt  (int __argc, char *__const * __argv,
			__const char *__opts)  ;
extern int opterr;
extern int optind;
extern int optopt;
extern char *optarg;





 


extern int __gethostname  (char *__name, size_t __len)  ;
extern int gethostname  (char *__name, size_t __len)  ;

 

extern int sethostname  (__const char *__name, size_t __len)  ;

 
extern long int gethostid  (void)  ;

 

extern int sethostid  (long int __id)  ;


 

extern size_t __getpagesize  (void)  ;
extern size_t getpagesize  (void)  ;


 

extern int __getdtablesize  (void)  ;
extern int getdtablesize  (void)  ;

 

extern int fdatasync  (int __fd)  ;

 
extern int fsync  (int __fd)  ;

 
extern int sync  (void)  ;


 


extern int vhangup  (void)  ;


 


extern int acct  (__const char *__name)  ;

 

extern int chroot  (__const char *__path)  ;

 

extern int swapon  (__const char *__path, int __swapflags)  ;

 
extern char *getusershell  (void)  ;
extern void endusershell  (void)  ;  
extern void setusershell  (void)  ;  

 

extern char *getpass  (__const char *__prompt)  ;




 



 








extern int lockf  (int __fd, int __cmd, __off_t __len)  ;




 
# 1 "/usr/include/sys/types.h" 1 3



# 1 "/usr/include/linux/types.h" 1 3




# 1 "/usr/include/asm/types.h" 1 3



typedef unsigned short umode_t;

 




typedef __signed__ char __s8;
typedef unsigned char __u8;

typedef __signed__ short __s16;
typedef unsigned short __u16;

typedef __signed__ int __s32;
typedef unsigned int __u32;


typedef __signed__ long long __s64;
typedef unsigned long long __u64;


 


# 42 "/usr/include/asm/types.h" 3



# 5 "/usr/include/linux/types.h" 2 3




typedef __kernel_fd_set		fd_set;
typedef __kernel_dev_t		dev_t;
typedef __kernel_ino_t		ino_t;
typedef __kernel_mode_t		mode_t;
typedef __kernel_nlink_t	nlink_t;
typedef __kernel_off_t		off_t;
typedef __kernel_pid_t		pid_t;
typedef __kernel_uid_t		uid_t;
typedef __kernel_gid_t		gid_t;
typedef __kernel_daddr_t	daddr_t;


typedef __kernel_loff_t		loff_t;


 















typedef __kernel_ptrdiff_t	ptrdiff_t;




typedef __kernel_time_t		time_t;




typedef __kernel_clock_t	clock_t;




typedef __kernel_caddr_t	caddr_t;


 
typedef unsigned char		u_char;
typedef unsigned short		u_short;
typedef unsigned int		u_int;
typedef unsigned long		u_long;

 
typedef unsigned char		unchar;
typedef unsigned short		ushort;
typedef unsigned int		uint;
typedef unsigned long		ulong;



 




struct ustat {
	__kernel_daddr_t	f_tfree;
	__kernel_ino_t		f_tinode;
	char			f_fname[6];
	char			f_fpack[6];
};


# 4 "/usr/include/sys/types.h" 2 3

# 1 "/usr/include/sys/bitypes.h" 1 3
 



 






















































 



 











	 



	typedef   char            int8_t;
	typedef unsigned char            u_int8_t;
	typedef short                     int16_t;
	typedef unsigned short          u_int16_t;
	typedef int                       int32_t;
	typedef unsigned int            u_int32_t;


	typedef long long                 int64_t;
	typedef unsigned long long      u_int64_t;




 
# 5 "/usr/include/sys/types.h" 2 3










 
typedef unsigned long fd_mask;


# 630 "/usr/include/unistd.h" 2 3


extern int	__brk  (void* __end_data_segment)  ;
extern int	brk  (void* __end_data_segment)  ;
extern void*	__sbrk  (ptrdiff_t __increment)  ;
extern void*	sbrk  (ptrdiff_t __increment)  ;

extern char*	crypt  (__const char *__key, __const char *__salt)  ;
extern void	encrypt  (char __block[64], int __edflag)  ;
extern void	setkey  (__const char *__key)  ;

extern int	ftruncate  (int __fildes, size_t __length)  ;
extern int	truncate  (__const char *__path, size_t __length)  ;

extern int	ioperm   (unsigned long __from,
			unsigned long __num, int __turn_on)  ;
extern int	iopl  (int __level)  ;

extern char*	mktemp  (char *__template)  ;
extern int	mkstemp  (char *__template)  ;

extern int	nice  (int __val)  ;

extern int	profil  (char *__buf, int __bufsiz, int __offset,
			int __scale)  ;

extern void	usleep  (unsigned long __usec)  ;



extern int	_xustat  (int __ver, dev_t __dev, struct ustat* __ubuf)  ;
static __inline__ int
ustat (dev_t __dev, struct ustat* __ubuf)
{
  return _xustat (1 , __dev, __ubuf);
}

extern int	idle  (void)  ;
extern int	reboot  (int __magic, int __magic_too,
			int __flag)  ;

extern int	swapoff  (__const char * __specialfile)  ;

extern int	uselib  (__const char *__filename)  ;

extern int	getdomainname  (char *__name, size_t __len)  ;
extern int	setdomainname  (__const char *__name, size_t __len)  ;
extern char*	realpath  (__const char *__path,
			char __resolved_path [])  ;












 


# 32 "g10.c" 2


# 1 "packet.h" 1
 






















# 1 "../include/types.h" 1
 


























  
  typedef unsigned char byte;
  















  
  


    typedef unsigned short u16;
  


  



  
  
    typedef unsigned int u32;
  




  





typedef struct string_list {
    struct string_list *next;
    char d[1];
} *STRLIST;



# 24 "packet.h" 2

# 1 "../include/iobuf.h" 1
 



































typedef struct iobuf_struct *IOBUF;

struct iobuf_struct {
    int usage;		  
    unsigned long nlimit;
    unsigned long nbytes;  
    unsigned long ntotal;  
    struct {
	size_t size;    
	size_t start;   
	size_t len;     
	byte *buf;
    } d;
    int filter_eof;
    int (*filter)( void *opaque, int control,
		   IOBUF chain, byte *buf, size_t *len);
    void *filter_ov;	 
    IOBUF chain;	 
    int no, subno;
    const char *desc;
    void *opaque;       
		        
    struct {
	size_t size;    
	size_t start;   
	size_t len;     
	byte *buf;
    } unget;
};

int iobuf_debug_mode;

IOBUF iobuf_alloc(int usage, size_t bufsize);
IOBUF iobuf_temp(void);
IOBUF iobuf_open( const char *fname );
IOBUF iobuf_create( const char *fname );
IOBUF iobuf_append( const char *fname );
IOBUF iobuf_openrw( const char *fname );
int   iobuf_close( IOBUF iobuf );
int   iobuf_cancel( IOBUF iobuf );

int iobuf_push_filter( IOBUF a, int (*f)(void *opaque, int control,
		       IOBUF chain, byte *buf, size_t *len), void *ov );
int iobuf_pop_filter( IOBUF a, int (*f)(void *opaque, int control,
		      IOBUF chain, byte *buf, size_t *len), void *ov );
int iobuf_flush(IOBUF a);
void iobuf_clear_eof(IOBUF a);

void iobuf_set_limit( IOBUF a, unsigned long nlimit );

ulong iobuf_tell( IOBUF a );
int   iobuf_seek( IOBUF a, ulong newpos );

int  iobuf_readbyte(IOBUF a);
int  iobuf_read(IOBUF a, byte *buf, unsigned buflen );
int  iobuf_peek(IOBUF a, byte *buf, unsigned buflen );
int  iobuf_writebyte(IOBUF a, unsigned c);
int  iobuf_write(IOBUF a, byte *buf, unsigned buflen );
int  iobuf_writestr(IOBUF a, const char *buf );

int  iobuf_write_temp( IOBUF a, IOBUF temp );
size_t iobuf_temp_to_buffer( IOBUF a, byte *buffer, size_t buflen );
void iobuf_unget_and_close_temp( IOBUF a, IOBUF temp );

u32 iobuf_get_filelength( IOBUF a );
const char *iobuf_get_fname( IOBUF a );

void iobuf_set_block_mode( IOBUF a, size_t n );
void iobuf_set_partial_block_mode( IOBUF a, size_t len );
int  iobuf_in_block_mode( IOBUF a );

 









 




 











# 25 "packet.h" 2

# 1 "../include/mpi.h" 1
 


































# 1 "../include/memory.h" 1
 






















# 49 "../include/memory.h"

void *m_alloc( size_t n );
void *m_alloc_clear( size_t n );
void *m_alloc_secure( size_t n );
void *m_alloc_secure_clear( size_t n );
void *m_realloc( void *a, size_t n );
void m_free( void *p );
void m_check( const void *a );
 
char *m_strdup( const char * a);


size_t m_size( const void *a );

 
void secmem_init( size_t npool );
void secmem_term( void );
void *secmem_malloc( size_t size );
void *secmem_realloc( void *a, size_t newsize );
void secmem_free( void *a );
int  m_is_secure( const void *p );
void secmem_dump_stats(void);
void secmem_set_flags( unsigned flags );
unsigned secmem_get_flags(void);





int memory_debug_mode;
int memory_stat_debug_mode;




# 36 "../include/mpi.h" 2




int mpi_debug_mode;



typedef unsigned long int mpi_limb_t;
typedef   __signed  long int mpi_limb_signed_t;

typedef struct mpi_struct {
    int alloced;     
    int nlimbs;      
    int nbits;	     
    int sign;	     
    int secure;      
    mpi_limb_t *d;   
} *MPI;






 

# 74 "../include/mpi.h"

  MPI mpi_alloc( unsigned nlimbs );
  MPI mpi_alloc_secure( unsigned nlimbs );
  void mpi_free( MPI a );
  void mpi_resize( MPI a, unsigned nlimbs );
  MPI  mpi_copy( MPI a );


void mpi_set_secure( MPI a );
void mpi_clear( MPI a );
void mpi_set( MPI w, MPI u);
void mpi_set_ui( MPI w, ulong u);
MPI  mpi_alloc_set_ui( unsigned long u);
void mpi_m_check( MPI a );
void mpi_swap( MPI a, MPI b);

 
int mpi_write( IOBUF out, MPI a );




  MPI mpi_read(IOBUF inp, unsigned *nread, int secure);

int mpi_fromstr(MPI val, const char *str);
int mpi_print( FILE *fp, MPI a, int mode );
u32 mpi_get_keyid( MPI a, u32 *keyid );
byte *mpi_get_buffer( MPI a, unsigned *nbytes, int *sign );
void  mpi_set_buffer( MPI a, const byte *buffer, unsigned nbytes, int sign );

 
void mpi_add_ui(MPI w, MPI u, ulong v );
void mpi_add(MPI w, MPI u, MPI v);
void mpi_addm(MPI w, MPI u, MPI v, MPI m);
void mpi_sub_ui(MPI w, MPI u, ulong v );
void mpi_sub( MPI w, MPI u, MPI v);
void mpi_subm( MPI w, MPI u, MPI v, MPI m);

 
void mpi_mul_ui(MPI w, MPI u, ulong v );
void mpi_mul_2exp( MPI w, MPI u, ulong cnt);
void mpi_mul( MPI w, MPI u, MPI v);
void mpi_mulm( MPI w, MPI u, MPI v, MPI m);

 
ulong mpi_fdiv_r_ui( MPI rem, MPI dividend, ulong divisor );
void  mpi_fdiv_r( MPI rem, MPI dividend, MPI divisor );
void  mpi_fdiv_q( MPI quot, MPI dividend, MPI divisor );
void  mpi_fdiv_qr( MPI quot, MPI rem, MPI dividend, MPI divisor );
void  mpi_tdiv_r( MPI rem, MPI num, MPI den);
void  mpi_tdiv_qr( MPI quot, MPI rem, MPI num, MPI den);
void  mpi_tdiv_q_2exp( MPI w, MPI u, unsigned count );
int   mpi_divisible_ui(MPI dividend, ulong divisor );

 
int mpi_gcd( MPI g, MPI a, MPI b );

 
void mpi_pow( MPI w, MPI u, MPI v);
void mpi_powm( MPI res, MPI base, MPI exp, MPI mod);

 
void mpi_mulpowm( MPI res, MPI *basearray, MPI *exparray, MPI mod);

 
int mpi_cmp_ui( MPI u, ulong v );
int mpi_cmp( MPI u, MPI v );

 
int mpi_getbyte( MPI a, unsigned index );
void mpi_putbyte( MPI a, unsigned index, int value );
unsigned mpi_trailing_zeros( MPI a );

 
unsigned mpi_get_nbits( MPI a );
int  mpi_test_bit( MPI a, unsigned n );
void mpi_set_bit( MPI a, unsigned n );
void mpi_set_highbit( MPI a, unsigned n );
void mpi_clear_highbit( MPI a, unsigned n );
void mpi_clear_bit( MPI a, unsigned n );
void mpi_set_bytes( MPI a, unsigned nbits, byte (*fnc)(int), int opaque );
void mpi_rshift( MPI x, MPI a, unsigned n );

 
void mpi_invm( MPI x, MPI u, MPI v );



# 26 "packet.h" 2

# 1 "../include/cipher.h" 1
 





























# 1 "../include/../cipher/md.h" 1
 























# 1 "../include/../cipher/rmd.h" 1
 























typedef struct {
    u32  h0,h1,h2,h3,h4;
    u32  nblocks;
    byte buf[64];
    int  count;
} RMD160_CONTEXT;


void rmd160_init( RMD160_CONTEXT *c );
void rmd160_write( RMD160_CONTEXT *hd, byte *inbuf, size_t inlen);
void rmd160_final(RMD160_CONTEXT *hd);
void rmd160_mixblock( RMD160_CONTEXT *hd, char *buffer );



# 25 "../include/../cipher/md.h" 2

# 1 "../include/../cipher/sha1.h" 1
 























typedef struct {
    u32  h0,h1,h2,h3,h4;
    u32  nblocks;
    byte buf[64];
    int  count;
} SHA1_CONTEXT;


void sha1_init( SHA1_CONTEXT *c );
void sha1_write( SHA1_CONTEXT *hd, byte *inbuf, size_t inlen);
void sha1_final( SHA1_CONTEXT *hd);



# 26 "../include/../cipher/md.h" 2

# 1 "../include/../cipher/md5.h" 1
 























typedef struct {
    u32 A,B,C,D;	   
    u32 total[2];
    u32  buflen;
    char buffer[128];
} MD5_CONTEXT;


void md5_init( MD5_CONTEXT *ctx );
void md5_write( MD5_CONTEXT *ctx, const void *buffer, size_t len);
void md5_final( MD5_CONTEXT *ctx);



# 27 "../include/../cipher/md.h" 2




typedef struct {
    int use_rmd160;
    RMD160_CONTEXT rmd160;
    int use_sha1;
    SHA1_CONTEXT sha1;
    int use_md5;
    MD5_CONTEXT md5;
    byte buffer[512 ];  
    int  bufcount;
    int  secure;
    FILE  *debug;
} *MD_HANDLE;









 
MD_HANDLE md_open( int algo, int secure );
void md_enable( MD_HANDLE hd, int algo );
MD_HANDLE md_copy( MD_HANDLE a );
void md_close(MD_HANDLE a);
void md_write( MD_HANDLE a, byte *inbuf, size_t inlen);
void md_final(MD_HANDLE a);
byte *md_read( MD_HANDLE a, int algo );
int md_get_algo( MD_HANDLE a );
int md_digest_length( int algo );
const byte *md_asn_oid( int algo, size_t *asnlen, size_t *mdlen );



# 31 "../include/cipher.h" 2




# 1 "../include/../cipher/blowfish.h" 1
 


























typedef struct {
    u32 s0[256];
    u32 s1[256];
    u32 s2[256];
    u32 s3[256];
    u32 p[16 +2];
    byte iv[8 ];
    byte eniv[8 ];
    int  count;
} BLOWFISH_context;

void blowfish_setkey( BLOWFISH_context *c, byte *key, unsigned keylen );
void blowfish_setiv( BLOWFISH_context *c, byte *iv );
void blowfish_encode( BLOWFISH_context *c, byte *outbuf, byte *inbuf,
						    unsigned nblocks );
void blowfish_decode( BLOWFISH_context *c, byte *outbuf, byte *inbuf,
						    unsigned nblocks );
void blowfish_encode_cfb( BLOWFISH_context *c, byte *outbuf,
					 byte *inbuf, unsigned nbytes);
void blowfish_decode_cfb( BLOWFISH_context *c, byte *outbuf,
					 byte *inbuf, unsigned nbytes);



# 35 "../include/cipher.h" 2

# 1 "../include/../cipher/elgamal.h" 1
 























typedef struct {
    MPI p;	     
    MPI g;	     
    MPI y;	     
} ELG_public_key;


typedef struct {
    MPI p;	     
    MPI g;	     
    MPI y;	     
    MPI x;	     
} ELG_secret_key;


void elg_free_public_key( ELG_public_key *pk );
void elg_free_secret_key( ELG_secret_key *sk );
void elg_generate( ELG_public_key *pk, ELG_secret_key *sk,
				unsigned nbits, MPI **factors );
int  elg_check_secret_key( ELG_secret_key *sk );
void elg_encrypt(MPI a, MPI b, MPI input, ELG_public_key *pkey );
void elg_decrypt(MPI output, MPI a, MPI b, ELG_secret_key *skey );
void elg_sign(MPI a, MPI b, MPI input, ELG_secret_key *skey);
int  elg_verify(MPI a, MPI b, MPI input, ELG_public_key *pkey);


# 36 "../include/cipher.h" 2

# 1 "../include/../cipher/dsa.h" 1
 























typedef struct {
    MPI p;	     
    MPI q;	     
    MPI g;	     
    MPI y;	     
} DSA_public_key;


typedef struct {
    MPI p;	     
    MPI q;	     
    MPI g;	     
    MPI y;	     
    MPI x;	     
} DSA_secret_key;


void dsa_free_public_key( DSA_public_key *pk );
void dsa_free_secret_key( DSA_secret_key *sk );
void dsa_generate( DSA_public_key *pk, DSA_secret_key *sk, unsigned nbits );
int  dsa_check_secret_key( DSA_secret_key *sk );
void dsa_sign(MPI r, MPI s, MPI input, DSA_secret_key *skey);
int  dsa_verify(MPI r, MPI s, MPI input, DSA_public_key *pkey);


# 37 "../include/cipher.h" 2

# 1 "../include/../cipher/random.h" 1
 























 
void secure_random_alloc(void);
int  quick_random_gen( int onoff );
void randomize_buffer( byte *buffer, size_t length, int level );
byte get_random_byte( int level );
byte *get_random_bits( size_t nbits, int level, int secure );
void add_randomness( const void *buffer, size_t length, int source );


 

void random_poll(void);
void fast_random_poll(void);



# 38 "../include/cipher.h" 2
























typedef struct {
    int algo;
    int keylen;
    byte key[20];  
} DEK;


int cipher_debug_mode;


 
int string_to_cipher_algo( const char *string );
int string_to_pubkey_algo( const char *string );
int string_to_digest_algo( const char *string );
const char * cipher_algo_to_string( int algo );
const char * pubkey_algo_to_string( int algo );
const char * digest_algo_to_string( int algo );
int check_cipher_algo( int algo );
int check_pubkey_algo( int algo );
int check_digest_algo( int algo );


 
extern ushort small_prime_numbers[];

 
MPI generate_secret_prime( unsigned nbits );
MPI generate_public_prime( unsigned nbits );
MPI generate_elg_prime( unsigned pbits, unsigned qbits, MPI g, MPI **factors );



# 27 "packet.h" 2

# 1 "filter.h" 1
 
























typedef struct {
    MD_HANDLE md;       
    size_t maxbuf_size;
} md_filter_context_t;

typedef struct {
    int status;
    int what;
    byte radbuf[4];
    int  idx, idx2;
    u32 crc;
    byte helpbuf[100];
    int  helpidx, helplen;
    unsigned empty;     
    int faked;
    int parse_state;
    int inp_checked;    
    int inp_bypass;     
    int inp_eof;
    const char *hdrlines;
} armor_filter_context_t;


typedef struct {
    int status;
    void *opaque;    
    byte *inbuf;
    unsigned inbufsize;
    byte *outbuf;
    unsigned outbufsize;
    int pgpmode;
} compress_filter_context_t;


typedef struct {
    DEK *dek;
    u32 datalen;
    BLOWFISH_context *bf_ctx;
    int header;
} cipher_filter_context_t;



typedef struct {
    int eof;
    size_t idx;
    size_t len;
    byte buf[256];
} text_filter_context_t;


 

 
int md_filter( void *opaque, int control, IOBUF a, byte *buf, size_t *ret_len);
void free_md_filter_context( md_filter_context_t *mfx );

 
int use_armor_filter( IOBUF a );
int armor_filter( void *opaque, int control,
		  IOBUF chain, byte *buf, size_t *ret_len);

 
int compress_filter( void *opaque, int control,
		     IOBUF chain, byte *buf, size_t *ret_len);

 
int cipher_filter( void *opaque, int control,
		   IOBUF chain, byte *buf, size_t *ret_len);

 
int text_filter( void *opaque, int control,
		 IOBUF chain, byte *buf, size_t *ret_len);




# 28 "packet.h" 2


typedef enum {
	PKT_NONE	   =0,
	PKT_PUBKEY_ENC	   =1,  
	PKT_SIGNATURE	   =2,  
	PKT_SESSION_KEY    =3,  
	PKT_ONEPASS_SIG    =4,  
	PKT_SECRET_CERT    =5,  
	PKT_PUBLIC_CERT    =6,  
	PKT_SECKEY_SUBCERT =7,  
	PKT_COMPRESSED	   =8,  
	PKT_ENCRYPTED	   =9,  
	PKT_MARKER	  =10,  
	PKT_PLAINTEXT	  =11,  
	PKT_RING_TRUST	  =12,  
	PKT_USER_ID	  =13,  
	PKT_PUBKEY_SUBCERT=14,  
	PKT_COMMENT	  =16   
} pkttype_t;

typedef struct packet_struct PACKET;

typedef struct {
    u32     keyid[2];	     
    byte    pubkey_algo;     
    union {
      struct {
	MPI  a, b;	     
      } elg;
      struct {
	MPI  rsa_integer;    
      } rsa;
    } d;
} PKT_pubkey_enc;


typedef struct {
    u32     keyid[2];	     
    byte    sig_class;	     
    byte    digest_algo;     
    byte    pubkey_algo;     
    byte    last;	     
} PKT_onepass_sig;


typedef struct {
    u32     keyid[2];	     
    ulong   local_id;	     
    u32     timestamp;	     
    byte    version;
    byte    sig_class;	     
    byte    pubkey_algo;     
			     
    byte digest_algo;	     
    byte *hashed_data;	     
    byte *unhashed_data;     
    byte digest_start[2];    
    union {
      struct {
	MPI  a, b;	       
      } elg;
      struct {
	MPI  r, s;	       
      } dsa;
      struct {
	MPI  rsa_integer;      
      } rsa;
    } d;
} PKT_signature;


typedef struct {
    u32     timestamp;	     
    u16     valid_days;      
    byte    hdrbytes;	     
    byte    version;
    byte    pubkey_algo;     
    ulong   local_id;	     
    union {
      struct {
	MPI p;		     
	MPI g;		     
	MPI y;		     
      } elg;
      struct {
	MPI p;		     
	MPI q;		     
	MPI g;		     
	MPI y;		     
      } dsa;
      struct {
	MPI rsa_n;	     
	MPI rsa_e;	     
      } rsa;
    } d;
} PKT_public_cert;

typedef struct {
    u32     timestamp;	     
    u16     valid_days;      
    byte    hdrbytes;	     
    byte    version;
    byte    pubkey_algo;     
    union {
      struct {
	MPI p;		     
	MPI g;		     
	MPI y;		     
	MPI x;		     
	u16 csum;	     
	byte is_protected;   
			     
	struct {
	    byte algo;	 
	    byte s2k;
	    byte hash;
	    byte salt[8];
	    byte count;
	    byte iv[8];  
	} protect;	     

      } elg;
      struct {
	MPI p;		     
	MPI q;		     
	MPI g;		     
	MPI y;		     
	MPI x;		     
	u16 csum;	     
	byte is_protected;   
			     
	struct {
	    byte algo;	 
	    byte s2k;
	    byte hash;
	    byte salt[8];
	    byte count;
	    byte iv[8];  
	} protect;	     

      } dsa;
      struct {
	MPI rsa_n;	     
	MPI rsa_e;	     
	MPI rsa_d;	     
	MPI rsa_p;	     
	MPI rsa_q;	     
	MPI rsa_u;	     
	u16 csum;	     
	byte is_protected;   
			     
	byte protect_algo;   
	union { 	     
	  struct {
	    byte iv[8];      
			     

	  } blowfish;
	} protect;
      } rsa;
    } d;
} PKT_secret_cert;


typedef struct {
    int  len;		   
    char data[1];
} PKT_comment;

typedef struct {
    int  len;		   
    char name[1];
} PKT_user_id;

typedef struct {
    u32  len;		   
    byte  algorithm;
    IOBUF buf;		   
} PKT_compressed;

typedef struct {
    u32  len;		   
    IOBUF buf;		   
} PKT_encrypted;

typedef struct {
    u32  len;		   
    IOBUF buf;		   
    int mode;
    u32 timestamp;
    int  namelen;
    char name[1];
} PKT_plaintext;

 
struct packet_struct {
    pkttype_t pkttype;
    union {
	void *generic;
	PKT_pubkey_enc	*pubkey_enc;	 
	PKT_onepass_sig *onepass_sig;	 
	PKT_signature	*signature;	 
	PKT_public_cert *public_cert;	 
	PKT_secret_cert *secret_cert;	 
	PKT_comment	*comment;	 
	PKT_user_id	*user_id;	 
	PKT_compressed	*compressed;	 
	PKT_encrypted	*encrypted;	 
	PKT_plaintext	*plaintext;	 
    } pkt;
};





 
int proc_packets( IOBUF a );
int proc_signature_packets( IOBUF a, STRLIST signedfiles );
int proc_encryption_packets( IOBUF a );
int list_packets( IOBUF a );

 
int set_packet_list_mode( int mode );
int search_packet( IOBUF inp, PACKET *pkt, int pkttype, ulong *retpos );
int parse_packet( IOBUF inp, PACKET *ret_pkt);
int copy_all_packets( IOBUF inp, IOBUF out );
int copy_some_packets( IOBUF inp, IOBUF out, ulong stopoff );
int skip_some_packets( IOBUF inp, unsigned n );

 
int build_packet( IOBUF inp, PACKET *pkt );
u32 calc_packet_length( PACKET *pkt );
void hash_public_cert( MD_HANDLE md, PKT_public_cert *pkc );

 
void free_pubkey_enc( PKT_pubkey_enc *enc );
void free_seckey_enc( PKT_signature *enc );
int  digest_algo_from_sig( PKT_signature *sig );
void release_public_cert_parts( PKT_public_cert *cert );
void free_public_cert( PKT_public_cert *cert );
void release_secret_cert_parts( PKT_secret_cert *cert );
void free_secret_cert( PKT_secret_cert *cert );
void free_user_id( PKT_user_id *uid );
void free_comment( PKT_comment *rem );
void free_packet( PACKET *pkt );
PKT_public_cert *copy_public_cert( PKT_public_cert *d, PKT_public_cert *s );
PKT_secret_cert *copy_secret_cert( PKT_secret_cert *d, PKT_secret_cert *s );
int cmp_public_certs( PKT_public_cert *a, PKT_public_cert *b );
int cmp_public_secret_cert( PKT_public_cert *pkc, PKT_secret_cert *skc );
int cmp_user_ids( PKT_user_id *a, PKT_user_id *b );


 
int signature_check( PKT_signature *sig, MD_HANDLE digest );

 
int is_secret_key_protected( PKT_secret_cert *cert );
int check_secret_key( PKT_secret_cert *cert );
int protect_secret_key( PKT_secret_cert *cert, DEK *dek );

 
int get_session_key( PKT_pubkey_enc *k, DEK *dek );

 
int handle_compressed( PKT_compressed *cd,
		       int (*callback)(IOBUF, void *), void *passthru );

 
int decrypt_data( PKT_encrypted *ed, DEK *dek );
int encrypt_data( PKT_encrypted *ed, DEK *dek );

 
int handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx );
int ask_for_detached_datafile( md_filter_context_t *mfx, const char *inname );

 
int write_comment( IOBUF out, const char *s );

 
int make_keysig_packet( PKT_signature **ret_sig, PKT_public_cert *pkc,
			PKT_user_id *uid, PKT_secret_cert *skc,
			int sigclass, int digest_algo );


# 34 "g10.c" 2



# 1 "../include/util.h" 1
 






















# 1 "../include/errors.h" 1
 






























































# 24 "../include/util.h" 2





typedef struct {
     int  *argc;	     
     char ***argv;	     
     unsigned flags;	     
     int err;		     
			     
     int r_opt; 	     
     int r_type;	     
     union {
	 int   ret_int;
	 long  ret_long;
	 ulong ret_ulong;
	 char *ret_str;
     } r;		     
     struct {
	 int index;
	 int inarg;
	 int stopped;
	 const char *last;
     } internal;	     
} ARGPARSE_ARGS;

typedef struct {
    int 	short_opt;
    const char *long_opt;
    unsigned flags;
    const char *description;  
} ARGPARSE_OPTS;

 
void log_set_name( const char *name );
const char *log_get_name(void);
void log_set_pid( int pid );
int  log_get_errorcount( int clear );
void log_hexdump( const char *text, char *buf, size_t len );
void log_mpidump( const char *text, MPI a );


  void log_bug( const char *fmt, ... )
			    __attribute__ ((noreturn, format (printf,1,2)));
  void log_bug0( const char *, int, const char * ) __attribute__ ((noreturn));
  void log_fatal( const char *fmt, ... )
			    __attribute__ ((noreturn, format (printf,1,2)));
  void log_error( const char *fmt, ... ) __attribute__ ((format (printf,1,2)));
  void log_info( const char *fmt, ... )  __attribute__ ((format (printf,1,2)));
  void log_debug( const char *fmt, ... ) __attribute__ ((format (printf,1,2)));
  
# 84 "../include/util.h"



 
const char * g10_errstr( int no );

 
int arg_parse( ARGPARSE_ARGS *arg, ARGPARSE_OPTS *opts);
int optfile_parse( FILE *fp, const char *filename, unsigned *lineno,
		   ARGPARSE_ARGS *arg, ARGPARSE_OPTS *opts);
void usage( int level );
const char *default_strusage( int level );


 
const char *strusage( int level );


 
char *make_filename( const char *first_part, ... );
const char *print_fname_stdin( const char *s );
const char *print_fname_stdout( const char *s );


 
u32 make_timestamp(void);
void print_string( FILE *fp, byte *p, size_t n, int delim );
int answer_is_yes( const char *s );

 
void free_strlist( STRLIST sl );

void add_to_strlist( STRLIST *list, const char *string );
STRLIST strlist_prev( STRLIST head, STRLIST node );
STRLIST strlist_last( STRLIST node );
char *memistr( char *buf, size_t buflen, const char *sub );
char *mem2str( char *, const void *, size_t);
char *trim_spaces( char *string );
int string_count_chr( const char *string, int c );






char *strlwr(char *a);





 

  






# 37 "g10.c" 2

# 1 "main.h" 1
 























# 1 "keydb.h" 1
 




























 







typedef struct kbnode_struct *KBNODE;
struct kbnode_struct {
    KBNODE next;
    PACKET *pkt;
    int flag;
    int private_flag;
};

 



struct keyblock_pos_struct {
    int   resno;      
    ulong offset;     
    unsigned count;   
    IOBUF  fp;	      
    PACKET *pkt;      
};
typedef struct keyblock_pos_struct KBPOS;

 
typedef struct pkc_list *PKC_LIST;
struct pkc_list {
    PKC_LIST next;
    PKT_public_cert *pkc;
    int mark;
};

 
typedef struct skc_list *SKC_LIST;
struct skc_list {
    SKC_LIST next;
    PKT_secret_cert *skc;
    int mark;
};

 

typedef struct pubkey_find_info *PUBKEY_FIND_INFO;
struct pubkey_find_info {
    u32  keyid[2];
    unsigned nbits;
    byte pubkey_algo;
    byte fingerprint[20];
    char userid[1];
};


 
void release_pkc_list( PKC_LIST pkc_list );
int  build_pkc_list( STRLIST remusr, PKC_LIST *ret_pkc_list );

 
void release_skc_list( SKC_LIST skc_list );
int  build_skc_list( STRLIST locusr, SKC_LIST *ret_skc_list, int unlock );

 
void set_passphrase_fd( int fd );
int  get_passphrase_fd(void);
DEK *get_passphrase_hash( u32 *keyid, char *text, byte *salt );
int make_dek_from_passphrase( DEK *dek, int mode, byte *salt );

 
void add_keyring( const char *name );
const char *get_keyring( int sequence );
void add_secret_keyring( const char *name );
void cache_public_cert( PKT_public_cert *pkc );
void cache_user_id( PKT_user_id *uid, u32 *keyid );
int get_pubkey( PKT_public_cert *pkc, u32 *keyid );
int get_pubkey_byname( PKT_public_cert *pkc, const char *name );
int get_seckey( PKT_secret_cert *skc, u32 *keyid );
int seckey_available( u32 *keyid );
int get_seckey_byname( PKT_secret_cert *skc, const char *name, int unlock );
int enum_secret_keys( void **context, PKT_secret_cert *skc );
char*get_user_id_string( u32 *keyid );
char*get_user_id( u32 *keyid, size_t *rn );

 
int pubkey_letter( int algo );
u32 keyid_from_skc( PKT_secret_cert *skc, u32 *keyid );
u32 keyid_from_pkc( PKT_public_cert *pkc, u32 *keyid );
u32 keyid_from_sig( PKT_signature *sig, u32 *keyid );
unsigned nbits_from_pkc( PKT_public_cert *pkc );
unsigned nbits_from_skc( PKT_secret_cert *skc );
const char *datestr_from_pkc( PKT_public_cert *pkc );
const char *datestr_from_skc( PKT_secret_cert *skc );
const char *datestr_from_sig( PKT_signature *sig );
byte *fingerprint_from_skc( PKT_secret_cert *skc, size_t *ret_len );
byte *fingerprint_from_pkc( PKT_public_cert *pkc, size_t *ret_len );

 
KBNODE new_kbnode( PACKET *pkt );
KBNODE clone_kbnode( KBNODE node );
void release_kbnode( KBNODE n );
void delete_kbnode( KBNODE node );
void add_kbnode( KBNODE root, KBNODE node );
void insert_kbnode( KBNODE root, KBNODE node, int pkttype );
KBNODE find_prev_kbnode( KBNODE root, KBNODE node, int pkttype );
KBNODE find_next_kbnode( KBNODE node, int pkttype );
KBNODE find_kbnode( KBNODE node, int pkttype );
KBNODE walk_kbnode( KBNODE root, KBNODE *context, int all );
void clear_kbnode_flags( KBNODE n );
int  commit_kbnode( KBNODE *root );

 
int add_keyblock_resource( const char *filename, int force, int secret );
const char *keyblock_resource_name( KBPOS *kbpos );
int get_keyblock_handle( const char *filename, int secret, KBPOS *kbpos );
int find_keyblock( PUBKEY_FIND_INFO info, KBPOS *kbpos );
int find_keyblock_byname( KBPOS *kbpos, const char *username );
int find_keyblock_bypkc( KBPOS *kbpos, PKT_public_cert *pkc );
int find_secret_keyblock_byname( KBPOS *kbpos, const char *username );
int lock_keyblock( KBPOS *kbpos );
void unlock_keyblock( KBPOS *kbpos );
int read_keyblock( KBPOS *kbpos, KBNODE *ret_root );
int enum_keyblocks( int mode, KBPOS *kbpos, KBNODE *ret_root );
int insert_keyblock( KBPOS *kbpos, KBNODE root );
int delete_keyblock( KBPOS *kbpos );
int update_keyblock( KBPOS *kbpos, KBNODE root );



# 25 "main.h" 2



typedef struct {
    int header_okay;
    PKC_LIST pkc_list;
    cipher_filter_context_t cfx;
} encrypt_filter_context_t;


 

  void g10_exit(int rc) __attribute__ ((noreturn));




 
int encode_symmetric( const char *filename );
int encode_store( const char *filename );
int encode_crypt( const char *filename, STRLIST remusr );
int encrypt_filter( void *opaque, int control,
		    IOBUF a, byte *buf, size_t *ret_len);


 
int sign_file( STRLIST filenames, int detached, STRLIST locusr,
	       int encrypt, STRLIST remusr, const char *outfile );
int clearsign_file( const char *fname, STRLIST locusr, const char *outfile );
int sign_key( const char *username, STRLIST locusr );
int edit_keysigs( const char *username );
int delete_key( const char *username, int secure );
int change_passphrase( const char *username );

 
int check_key_signature( KBNODE root, KBNODE node, int *is_selfsig );

 
void generate_keypair(void);

 
int overwrite_filep( const char *fname );
IOBUF open_outfile( const char *fname, int mode );
IOBUF open_sigfile( const char *iname );

 
void make_session_key( DEK *dek );
MPI encode_session_key( DEK *dek, unsigned nbits );
MPI encode_md_value( MD_HANDLE md, unsigned nbits );

 
KBNODE make_comment_node( const char *s );
KBNODE make_mpi_comment_node( const char *s, MPI a );

 
void g10_elg_encrypt( PKT_public_cert *pkc, PKT_pubkey_enc *enc, DEK *dek );
void g10_elg_sign( PKT_secret_cert *skc, PKT_signature *sig,
					 MD_HANDLE md, int digest_algo );

 
void g10_dsa_sign( PKT_secret_cert *skc, PKT_signature *sig,
					 MD_HANDLE md, int digest_algo );

 
void g10_rsa_encrypt( PKT_public_cert *pkc, PKT_pubkey_enc *enc, DEK *dek );
void g10_rsa_sign( PKT_secret_cert *skc, PKT_signature *sig,
					 MD_HANDLE md, int digest_algo );

 
int import_pubkeys( const char *filename );
 
int export_pubkeys( STRLIST users );
 
int dearmor_file( const char *fname );
int enarmor_file( const char *fname );

 
int gen_revoke( const char *uname );

 
void std_key_list( int nnames, char **names );

 
int verify_signatures( int nfiles, char **files );

 
int decrypt_message( const char *filename );

 
int hash_datafiles( MD_HANDLE md, STRLIST files, int textmode );

 
void init_signals(void);


# 38 "g10.c" 2

# 1 "options.h" 1
 





















struct {
    int verbose;
    unsigned debug;
    int armor;
    int compress;
    char *outfile;
    int textmode;
    int batch;	     
    int answer_yes;  
    int answer_no;   
    int check_sigs;  
    int with_colons;
    int fingerprint;  
    int list_sigs;    
    int no_armor;
    int list_packets;  
    int def_cipher_algo;
    int def_pubkey_algo;
    int def_digest_algo;
    int no_comment;
    int marginals_needed;
    int completes_needed;
    const char *homedir;
    int reserved13;
    int reserved14;
    int reserved15;
} opt;





				 















# 39 "g10.c" 2





# 1 "trustdb.h" 1
 























 








 



 
void list_trustdb(const char *username);
void list_trust_path( int max_depth, const char *username );
int init_trustdb( int level, const char *dbname );
int check_trust( PKT_public_cert *pkc, unsigned *r_trustlevel );
int enum_trust_web( void **context, ulong *lid );
int get_ownertrust( ulong lid, unsigned *r_otrust );
int keyid_from_trustdb( ulong lid, u32 *keyid );
int query_trust_record( PKT_public_cert *pkc );
int insert_trust_record( PKT_public_cert *pkc );
int update_ownertrust( ulong lid, unsigned new_trust );
int verify_private_data(void);
int sign_private_data(void);


# 44 "g10.c" 2

# 1 "../include/ttyio.h" 1
 





















void tty_printf( const char *fmt, ... );
void tty_print_string( byte *p, size_t n );
char *tty_get( const char *prompt );
char *tty_get_hidden( const char *prompt );
void tty_kill_prompt(void);



# 45 "g10.c" 2

# 1 "../include/i18n.h" 1
 























  
# 1 "/usr/include/locale.h" 1 3
 

















 








 

 










 
struct lconv
{
   

  char *decimal_point;		 
  char *thousands_sep;		 
   




  char *grouping;

   

   

  char *int_curr_symbol;
  char *currency_symbol;	 
  char *mon_decimal_point;	 
  char *mon_thousands_sep;	 
  char *mon_grouping;		 
  char *positive_sign;		 
  char *negative_sign;		 
  char int_frac_digits;		 
  char frac_digits;		 
   
  char p_cs_precedes;
   
  char p_sep_by_space;
   
  char n_cs_precedes;
   
  char n_sep_by_space;
   





  char p_sign_posn;
  char n_sign_posn;
};


 
extern char *setlocale  (int __category, __const char *__locale)  ;

 
extern struct lconv *localeconv  (void)  ;

 


# 25 "../include/i18n.h" 2



# 36 "../include/i18n.h"

  
  



# 46 "g10.c" 2

# 1 "status.h" 1
 



































 
void set_status_fd( int fd );
void write_status( int no );
void write_status_text( int no, const char *text);



# 47 "g10.c" 2







static ARGPARSE_OPTS opts[] = {

    { 300, ((void *)0) , 0, ( "\vCommands:\n " )  },

  
# 67 "g10.c"

    { 551, "list-keys", 0, ( "list keys" ) },
    { 552, "list-sigs", 0, ( "list keys and signatures" ) },
    { 508, "check-sigs",0, ( "check key signatures" ) },
    { 515, "fingerprint", 0, ( "list keys and fingerprints" ) },
  







    { 537, "export"          , 0, ( "export keys" )  },
    { 530, "import",      0     , ( "import/merge keys" ) },
    { 521, "list-packets",0,( "list only the sequence of packets" ) },
  
    { 546, "dearmor", 0, ( "De-Armor a file or stdin" )  },
    { 547, "enarmor", 0, ( "En-Armor a file or stdin" )  },
    { 516, "print-mds" , 0, ( "print all message digests" ) },
    { 513, "gen-prime" , 0, "\r" },
    { 548, "gen-random" , 0, "\r" },
  

    { 301, ((void *)0) , 0, ( "\v\nOptions:\n " )  },

  






    { 'o', "output",    2, ( "use as output file" ) },
    { 'v', "verbose",   0, ( "verbose" )  },
    { 'n', "dry-run",   0, ( "don't make any changes" )  },
    { 500, "batch",     0, ( "batch mode: never ask" ) },
    { 501, "yes",       0, ( "assume yes on most questions" ) },
    { 502, "no",        0, ( "assume no on most questions" ) },
    { 509, "keyring"   ,2, ( "add this keyring to the list of keyrings" ) },
    { 517, "secret-keyring" ,2, ( "add this secret keyring to the list" ) },
    { 518, "options"   , 2, ( "read options from file" ) },

    { 510, "debug"     ,4|16, ( "set debugging flags" ) },
    { 511, "debug-all" ,0, ( "enable full debugging" ) },
    { 512, "status-fd" ,1, ( "write status info to this fd" )  },
    { 534, "no-comment", 0,   ( "do not write comment packets" ) },
    { 535, "completes-needed", 1, ( "(default is 1)" ) },
    { 536, "marginals-needed", 1, ( "(default is 3)" ) },
  





  








   
  
    { 514, "test"      , 0, "\r" },
    { 531, "list-trustdb",0 , "\r"},
    { 533, "list-trust-path",0, "\r"},
  
  





    { 532, "quick-random", 0, "\r"},
    { 526, "no-verbose", 0, "\r"},
    { 538, "trustdb-name", 2, "\r" },
    { 540, "no-secmem-warning", 0, "\r" },  
    { 519, "no-armor",   0, "\r"},
    { 520, "no-default-keyring", 0, "\r" },
    { 522, "no-greeting", 0, "\r" },
    { 541, "no-operation", 0, "\r" },       
    { 543, "no-options", 0, "\r" },  
    { 544, "homedir", 2, "\r" },    
    { 545, "no-batch", 0, "\r" },
    { 549, "with-colons", 0, "\r"},
    { 551, "list-key", 0, "\r" },  
    { 552, "list-sig", 0, "\r" },  
    { 508, "check-sig",0, "\r" },  


{0} };




enum cmd_values { aNull = 0,
    aSym, aStore, aEncr, aKeygen, aSign, aSignEncr,
    aSignKey, aClearsign, aListPackets, aEditSig, aDeleteKey, aDeleteSecretKey,
    aKMode, aKModeC, aChangePass, aImport, aVerify, aDecrypt, aListKeys,
    aListSigs,
    aExport, aCheckKeys, aGenRevoke, aPrimegen, aPrintMDs,
    aListTrustDB, aListTrustPath, aDeArmor, aEnArmor, aGenRandom, aTest,
aNOP };


static char *build_list( const char *text,
			 const char *(*mapf)(int), int (*chkf)(int) );
static void set_cmd( enum cmd_values *ret_cmd,
			enum cmd_values new_cmd );

static void print_hex( byte *p, size_t n );
static void print_mds( const char *fname );
static void do_test(int);


const char *
strusage( int level )
{
  static char *digests, *pubkeys, *ciphers;
    const char *p;
    switch( level ) {
      case 11: p =
	  
	    "gpgm (GNUPG)";
	  


	break;
      case 13: p = "0.2.12a" ; break;
      case 17: p = "Linux" ; break;
      case 19: p =
	    ( "Please report bugs to <gnupg-bugs@isil.d.shuttle.de>.\n" ) ;
	break;
      case 1:
      case 40:	p =
	  
	    ( "Usage: gpgm [options] [files] (-h for help)" ) ;
	  


	break;
      case 41:	p =
	  
	    ( "Syntax: gpgm [options] [files]\n"
	      "GNUPG maintenance utility\n" ) ;
	  




	break;

      case 31: p = "\n"; break;
      case 32:
	if( !ciphers )
	    ciphers = build_list("Supported ciphers: ", cipher_algo_to_string,
							check_cipher_algo );
	p = ciphers;
	break;
      case 33:
	if( !pubkeys )
	    pubkeys = build_list("Supported pubkeys: ", pubkey_algo_to_string,
							check_pubkey_algo );
	p = pubkeys;
	break;
      case 34:
	if( !digests )
	    digests = build_list("Supported digests: ", digest_algo_to_string,
							check_digest_algo );
	p = digests;
	break;

      default:	p = default_strusage(level);
    }
    return p;
}


static char *
build_list( const char *text, const char * (*mapf)(int), int (*chkf)(int) )
{
    int i;
    const char *s;
    size_t n=strlen(text)+2;
    char *list, *p;

    for(i=1; i < 100; i++ )
	if( !chkf(i) && (s=mapf(i)) )
	    n += strlen(s) + 2;
    list = m_alloc( 21 + n ); *list = 0;
    for(p= ((void *)0) , i=1; i < 100; i++ ) {
	if( !chkf(i) && (s=mapf(i)) ) {
	    if( !p )
		p = stpcpy( list, text );
	    else
		p = stpcpy( p, ", ");
	    p = stpcpy(p, s );
	}
    }
    if( p )
	p = stpcpy(p, "\n" );
    return list;
}


static void
i18n_init(void)
{
  
# 286 "g10.c"

}

static void
wrong_args( const char *text)
{
  
    fputs(( "usage: gpgm [options] " ) ,((_IO_FILE*)(&_IO_stderr_))  );
  


    fputs(text,((_IO_FILE*)(&_IO_stderr_))  );
    (((   ((_IO_FILE*)(&_IO_stderr_))    )->_IO_write_ptr >= (   ((_IO_FILE*)(&_IO_stderr_))    )->_IO_write_end) ? __overflow(   ((_IO_FILE*)(&_IO_stderr_))    , (unsigned char)(  '\n'  )) : (unsigned char)(*(   ((_IO_FILE*)(&_IO_stderr_))    )->_IO_write_ptr++ = (  '\n'  )))  ;
    g10_exit(2);
}

static void
set_debug(void)
{
    if( opt.debug & 32  )
	memory_debug_mode = 1;
    if( opt.debug & 128  )
	memory_stat_debug_mode = 1;
    if( opt.debug & 2  )
	mpi_debug_mode = 1;
    if( opt.debug & 4  )
	cipher_debug_mode = 1;
    if( opt.debug & 16  )
	iobuf_debug_mode = 1;
}


static void
set_cmd( enum cmd_values *ret_cmd, enum cmd_values new_cmd )
{
    enum cmd_values cmd = *ret_cmd;

    if( !cmd || cmd == new_cmd )
	cmd = new_cmd;
    else if( cmd == aSign && new_cmd == aEncr )
	cmd = aSignEncr;
    else if( cmd == aEncr && new_cmd == aSign )
	cmd = aSignEncr;
    else if( cmd == aKMode && new_cmd == aSym )
	cmd = aKModeC;
    else if(	( cmd == aSign	   && new_cmd == aClearsign )
	     || ( cmd == aClearsign && new_cmd == aSign )  )
	cmd = aClearsign;
    else {
	log_error(( "conflicting commands\n" ) );
	g10_exit(2);
    }

    *ret_cmd = cmd;
}



static void
check_opts(void)
{
    if( !opt.def_cipher_algo || check_cipher_algo(opt.def_cipher_algo) )
	log_error(( "selected cipher algorithm is invalid\n" ) );
    if( !opt.def_pubkey_algo || check_pubkey_algo(opt.def_pubkey_algo) )
	log_error(( "selected pubkey algorithm is invalid\n" ) );
    if( !opt.def_digest_algo || check_digest_algo(opt.def_digest_algo) )
	log_error(( "selected digest algorithm is invalid\n" ) );
    if( opt.completes_needed < 1 )
	log_error(( "completes-needed must be greater than 0\n" ) );
    if( opt.marginals_needed < 2 )
	log_error(( "marginals-needed must be greater than 1\n" ) );
}




void
main( int argc, char **argv )
{
    ARGPARSE_ARGS pargs;
    IOBUF a;
    int rc=0;
    int orig_argc;
    char **orig_argv;
    const char *fname;
    STRLIST sl, remusr= ((void *)0) , locusr= ((void *)0) ;
    int nrings=0, sec_nrings=0;
    armor_filter_context_t afx;
    int detached_sig = 0;
    FILE *configfp = ((void *)0) ;
    char *configname = ((void *)0) ;
    unsigned configlineno;
    int parse_debug = 0;
    int default_config =1;
    int errors=0;
    int default_keyring = 1;
    int greeting = 1;
    enum cmd_values cmd = 0;
    const char *trustdb_name = ((void *)0) ;

  
    secmem_init( 0 );	    
    log_set_name("gpgm");
  
# 397 "g10.c"

    i18n_init();
    opt.compress = -1;  
    opt.def_cipher_algo = 42 ;
    opt.def_pubkey_algo = 16 ;
    opt.def_digest_algo = 3 ;
    opt.completes_needed = 1;
    opt.marginals_needed = 3;
    opt.homedir = getenv("GNUPGHOME");
    if( !opt.homedir || !*opt.homedir ) {
      


	opt.homedir = "~/.gnupg";
      
    }

     
    orig_argc = argc;
    orig_argv = argv;
    pargs.argc = &argc;
    pargs.argv = &argv;
    pargs.flags=  1;   
    while( arg_parse( &pargs, opts) ) {
	if( pargs.r_opt == 510 || pargs.r_opt == 511 )
	    parse_debug++;
	else if( pargs.r_opt == 518 ) {
	     


	    default_config = 0;
	}
	else if( pargs.r_opt == 543 )
	    default_config = 0;  
	else if( pargs.r_opt == 544 )
	    opt.homedir = pargs.r.ret_str;
    }

    if( default_config )
	configname = make_filename(opt.homedir, "options", ((void *)0)  );

    argc = orig_argc;
    argv = orig_argv;
    pargs.argc = &argc;
    pargs.argv = &argv;
    pargs.flags=  1;   
  next_pass:
    if( configname ) {
	configlineno = 0;
	configfp = fopen( configname, "r" );
	if( !configfp ) {
	    if( default_config ) {
		if( parse_debug )
		    log_info(( "note: no default option file '%s'\n" ) ,
							    configname );
	    }
	    else {
		log_error(( "option file '%s': %s\n" ) ,
				    configname, strerror(errno) );
		g10_exit(1);
	    }
	    m_free(configname); configname = ((void *)0) ;
	}
	if( parse_debug && configname )
	    log_info(( "reading options from '%s'\n" ) , configname );
	default_config = 0;
    }

    while( optfile_parse( configfp, configname, &configlineno,
						&pargs, opts) ) {
	switch( pargs.r_opt ) {

	
# 511 "g10.c"


	
	  case 513: set_cmd( &cmd, aPrimegen); break;
	  case 514: set_cmd( &cmd, aTest); break;
	  case 516: set_cmd( &cmd, aPrintMDs); break;
	  case 531: set_cmd( &cmd, aListTrustDB); break;
	  case 533: set_cmd( &cmd, aListTrustPath); break;
	  case 540: break;  
	  case 546: set_cmd( &cmd, aDeArmor); break;
	  case 547: set_cmd( &cmd, aEnArmor); break;
	  case 548: set_cmd( &cmd, aGenRandom); break;
	

	  case 'o': opt.outfile = pargs.r.ret_str; break;
	  case 'v': opt.verbose++; opt.list_sigs=1; break;
	  case 'k': set_cmd( &cmd, aKMode ); break;

	  case 500: opt.batch = 1; greeting = 0; break;
	  case 501: opt.answer_yes = 1; break;
	  case 502: opt.answer_no = 1; break;
	  case 508: set_cmd( &cmd, aCheckKeys); break;
	  case 509: add_keyring(pargs.r.ret_str); nrings++; break;
	  case 510: opt.debug |= pargs.r.ret_ulong; break;
	  case 511: opt.debug = ~0; break;
	  case 512: set_status_fd( pargs.r.ret_int ); break;
	  case 515: opt.fingerprint = 1; break;
	  case 517: add_secret_keyring(pargs.r.ret_str); sec_nrings++; break;
	  case 518:
	     
	    if( !configfp ) {
		m_free(configname);
		configname = m_strdup(pargs.r.ret_str);
		goto next_pass;
	    }
	    break;
	  case 519: opt.no_armor=1; opt.armor=0; break;
	  case 520: default_keyring = 0; break;
	  case 521: set_cmd( &cmd, aListPackets); break;
	  case 522: greeting = 0; break;
	  case 526: opt.verbose = 0; opt.list_sigs=0; break;
	  case 530: set_cmd( &cmd, aImport); break;
	  case 532: quick_random_gen(1); break;
	  case 534: opt.no_comment=1; break;
	  case 535: opt.completes_needed = pargs.r.ret_int; break;
	  case 536: opt.marginals_needed = pargs.r.ret_int; break;
	  case 537: set_cmd( &cmd, aExport); break;
	  case 538: trustdb_name = pargs.r.ret_str; break;
	  case 541: set_cmd( &cmd, aNOP); break;
	  case 543: break;  
	  case 544: opt.homedir = pargs.r.ret_str; break;
	  case 545: opt.batch = 0; break;
	  case 549: opt.with_colons=':'; break;
	  case 551: set_cmd( &cmd, aListKeys); break;
	  case 552: set_cmd( &cmd, aListSigs); break;
	  default : errors++; pargs.err = configfp? 1:2; break;
	}
    }
    if( configfp ) {
	fclose( configfp );
	configfp = ((void *)0) ;
	m_free(configname); configname = ((void *)0) ;
	goto next_pass;
    }
    m_free( configname ); configname = ((void *)0) ;
    check_opts();
    if( log_get_errorcount(0) )
	g10_exit(2);

    if( greeting ) {
	tty_printf("%s %s; %s\n", strusage(11), strusage(13), strusage(14) );
	tty_printf("%s\n", strusage(15) );
    }

  





     

    set_debug();
    if( !cmd && opt.fingerprint )
	set_cmd( &cmd, aListKeys);

    if( cmd == aKMode || cmd == aKModeC ) {  
	if( cmd == aKModeC ) {
	    opt.fingerprint = 1;
	    cmd = aKMode;
	}
	opt.list_sigs = 0;
	if( opt.verbose > 2 )
	    opt.check_sigs++;
	if( opt.verbose > 1 )
	    opt.list_sigs++;

	opt.verbose = opt.verbose > 1;
    }


     
    if( opt.textmode && !detached_sig && opt.armor && cmd == aSign )
	cmd = aClearsign;

    if( opt.verbose > 1 )
	set_packet_list_mode(1);

     

    if( cmd != aDeArmor && cmd != aEnArmor
	&& !(cmd == aKMode && argc == 2 ) ) {
	if( !sec_nrings || default_keyring )   
	    add_secret_keyring("secring.gpg");
	if( !nrings || default_keyring )   
	    add_keyring("pubring.gpg");
    }

    if( argc )
	fname = *argv;
    else {
	fname = ((void *)0) ;
	if( get_passphrase_fd() == 0 ) {
	     



	     
	}
    }

    switch( cmd ) {
      case aPrimegen:
      case aPrintMDs:
      case aGenRandom:
      case aDeArmor:
      case aEnArmor:
	break;
      case aKMode:
      case aListKeys:
      case aCheckKeys:
	break;
      case aListTrustDB: rc = init_trustdb( argc? 1:0, trustdb_name ); break;
      default: rc = init_trustdb(1, trustdb_name ); break;
    }
    if( rc )
	log_error(( "failed to initialize the TrustDB: %s\n" ) , g10_errstr(rc));


    switch( cmd ) {
      case aStore:  
	if( argc > 1 )
	    wrong_args(( "--store [filename]" ) );
	if( (rc = encode_store(fname)) )
	    log_error("%s: store failed: %s\n",
				 print_fname_stdin(fname), g10_errstr(rc) );
	break;
    
# 772 "g10.c"


      case aCheckKeys:
	opt.check_sigs = 1;
      case aListSigs:
	opt.list_sigs = 1;
      case aListKeys:
	std_key_list( argc, argv );
	break;

      case aKMode:  
	if( argc < 2 )	 
	    std_key_list( (argc && **argv)? 1:0, argv );
	else if( argc == 2 ) {  
	    if( access( argv[1], 4  ) ) {
		log_error(( "can't open %s: %s\n" ) ,
			       print_fname_stdin(argv[1]), strerror(errno));
	    }
	    else {
		 

		add_keyring( argv[1] );
		std_key_list( **argv?1:0, argv );
	    }
	}
	else
	    wrong_args(( "-k[v][v][v][c] [userid] [keyring]" )  );
	break;

    







      case aImport:
	if( !argc  ) {
	    rc = import_pubkeys( ((void *)0)  );
	    if( rc )
		log_error("import failed: %s\n", g10_errstr(rc) );
	}
	for( ; argc; argc--, argv++ ) {
	    rc = import_pubkeys( *argv );
	    if( rc )
		log_error("import from '%s' failed: %s\n",
						*argv, g10_errstr(rc) );
	}
	break;

      case aExport:
	sl = ((void *)0) ;
	for( ; argc; argc--, argv++ )
	    add_to_strlist( &sl, *argv );
	export_pubkeys( sl );
	free_strlist(sl);
	break;

    







    
      case aDeArmor:
	if( argc > 1 )
	    wrong_args("--dearmor [file]");
	rc = dearmor_file( argc? *argv: ((void *)0)  );
	if( rc )
	    log_error(( "dearmoring failed: %s\n" ) , g10_errstr(rc));
	break;

      case aEnArmor:
	if( argc > 1 )
	    wrong_args("--enarmor [file]");
	rc = enarmor_file( argc? *argv: ((void *)0)  );
	if( rc )
	    log_error(( "enarmoring failed: %s\n" ) , g10_errstr(rc));
	break;


      case aPrimegen:
	if( argc == 1 ) {
	    mpi_print( ((_IO_FILE*)(&_IO_stdout_))  , generate_public_prime( atoi(argv[0]) ), 1);
	    (((    ((_IO_FILE*)(&_IO_stdout_))    )->_IO_write_ptr >= (    ((_IO_FILE*)(&_IO_stdout_))    )->_IO_write_end) ? __overflow(    ((_IO_FILE*)(&_IO_stdout_))    , (unsigned char)(   '\n'   )) : (unsigned char)(*(    ((_IO_FILE*)(&_IO_stdout_))    )->_IO_write_ptr++ = (   '\n'   )))   ;
	}
	else if( argc == 2 ) {
	    mpi_print( ((_IO_FILE*)(&_IO_stdout_))  , generate_elg_prime( atoi(argv[0]),
						   atoi(argv[1]), ((void *)0) ,((void *)0)  ), 1);
	    (((    ((_IO_FILE*)(&_IO_stdout_))    )->_IO_write_ptr >= (    ((_IO_FILE*)(&_IO_stdout_))    )->_IO_write_end) ? __overflow(    ((_IO_FILE*)(&_IO_stdout_))    , (unsigned char)(   '\n'   )) : (unsigned char)(*(    ((_IO_FILE*)(&_IO_stdout_))    )->_IO_write_ptr++ = (   '\n'   )))   ;
	}
	else if( argc == 3 ) {
	    MPI g = mpi_alloc(1);
	    mpi_print( ((_IO_FILE*)(&_IO_stdout_))  , generate_elg_prime( atoi(argv[0]),
						   atoi(argv[1]), g, ((void *)0)  ), 1);
	    printf("\nGenerator: ");
	    mpi_print( ((_IO_FILE*)(&_IO_stdout_))  , g, 1 );
	    (((    ((_IO_FILE*)(&_IO_stdout_))    )->_IO_write_ptr >= (    ((_IO_FILE*)(&_IO_stdout_))    )->_IO_write_end) ? __overflow(    ((_IO_FILE*)(&_IO_stdout_))    , (unsigned char)(   '\n'   )) : (unsigned char)(*(    ((_IO_FILE*)(&_IO_stdout_))    )->_IO_write_ptr++ = (   '\n'   )))   ;
	    mpi_free(g);
	}
	else
	    usage(1);
	break;

      case aGenRandom:
	if( argc < 1 || argc > 2 )
	    wrong_args("--gen-random level [hex]");
	{
	    int level = atoi(*argv);
	    for(;;) {
		int c = get_random_byte(level);
		if( argc == 1 ) {
		    printf("%02x", c );
		    fflush(((_IO_FILE*)(&_IO_stdout_))  );
		}
		else
		    (((    ((_IO_FILE*)(&_IO_stdout_))    )->_IO_write_ptr >= (    ((_IO_FILE*)(&_IO_stdout_))    )->_IO_write_end) ? __overflow(    ((_IO_FILE*)(&_IO_stdout_))    , (unsigned char)(   c&0xff   )) : (unsigned char)(*(    ((_IO_FILE*)(&_IO_stdout_))    )->_IO_write_ptr++ = (   c&0xff   )))   ;
	    }
	}
	break;

      case aPrintMDs:
	if( !argc )
	    print_mds(((void *)0) );
	else {
	    for(; argc; argc--, argv++ )
		print_mds(*argv);
	}
	break;

      case aTest: do_test( argc? atoi(*argv): 1 ); break;

      case aListTrustDB:
	if( !argc )
	    list_trustdb(((void *)0) );
	else {
	    for( ; argc; argc--, argv++ )
		list_trustdb( *argv );
	}
	break;

      case aListTrustPath:
	if( argc != 2 )
	    wrong_args("--list-trust-path [-- -]<maxdepth> <username>");
	list_trust_path( atoi(*argv), argv[1] );
	break;

     


      case aNOP:
	break;

      case aListPackets:
	opt.list_packets=1;
      default:
	 
	if( argc > 1 )
	    wrong_args(( "[filename]" ) );
	if( !(a = iobuf_open(fname)) )
	    log_error(( "can't open '%s'\n" ) , print_fname_stdin(fname));
	else {
	    if( !opt.no_armor ) {
		if( use_armor_filter( a ) ) {
		    memset( &afx, 0, sizeof afx);
		    iobuf_push_filter( a, armor_filter, &afx );
		}
	    }
	    if( cmd == aListPackets ) {
		set_packet_list_mode(1);
		opt.list_packets=1;
	    }
	    proc_packets( a );
	    iobuf_close(a);
	}
	break;
    }

     
    do { free_strlist(( remusr )); ( remusr ) = ((void *)0)  ; } while(0) ;
    do { free_strlist(( locusr )); ( locusr ) = ((void *)0)  ; } while(0) ;
    g10_exit(0);
}


void
g10_exit( int rc )
{
    if( opt.debug )
	secmem_dump_stats();
    secmem_term();
    rc = rc? rc : log_get_errorcount(0)? 2:0;
     
    exit(rc );
}


static void
print_hex( byte *p, size_t n )
{
    int i;

    if( n == 20 ) {
	for(i=0; i < n ; i++, i++, p += 2 ) {
	    if( i == 10 )
		(((    ((_IO_FILE*)(&_IO_stdout_))    )->_IO_write_ptr >= (    ((_IO_FILE*)(&_IO_stdout_))    )->_IO_write_end) ? __overflow(    ((_IO_FILE*)(&_IO_stdout_))    , (unsigned char)(   ' '   )) : (unsigned char)(*(    ((_IO_FILE*)(&_IO_stdout_))    )->_IO_write_ptr++ = (   ' '   )))   ;
	    printf(" %02X%02X", *p, p[1] );
	}
    }
    else {
	for(i=0; i < n ; i++, p++ ) {
	    if( i && !(i%8) )
		(((    ((_IO_FILE*)(&_IO_stdout_))    )->_IO_write_ptr >= (    ((_IO_FILE*)(&_IO_stdout_))    )->_IO_write_end) ? __overflow(    ((_IO_FILE*)(&_IO_stdout_))    , (unsigned char)(   ' '   )) : (unsigned char)(*(    ((_IO_FILE*)(&_IO_stdout_))    )->_IO_write_ptr++ = (   ' '   )))   ;
	    printf(" %02X", *p );
	}
    }
}

static void
print_mds( const char *fname )
{
    FILE *fp;
    char buf[1024];
    size_t n;
    MD_HANDLE md;

    if( !fname ) {
	fp = ((_IO_FILE*)(&_IO_stdin_))  ;
	fname = "[stdin]";
    }
    else
	fp = fopen( fname, "rb" );
    if( !fp ) {
	log_error("%s: %s\n", fname, strerror(errno) );
	return;
    }

    md = md_open( 1 , 0 );
    md_enable( md, 3  );
    md_enable( md, 2  );

    while( (n=fread( buf, 1, (sizeof( buf )/sizeof(( buf )[0])) , fp )) )
	md_write( md, buf, n );
    if( ferror(fp) )
	log_error("%s: %s\n", fname, strerror(errno) );
    else {
	md_final(md);
	printf(  "%s:    MD5 =", fname ); print_hex(md_read(md, 1 ), 16 );
	printf("\n%s: RMD160 =", fname ); print_hex(md_read(md, 3 ), 20 );
	printf("\n%s:   SHA1 =", fname ); print_hex(md_read(md, 2 ), 20 );
	(((    ((_IO_FILE*)(&_IO_stdout_))    )->_IO_write_ptr >= (    ((_IO_FILE*)(&_IO_stdout_))    )->_IO_write_end) ? __overflow(    ((_IO_FILE*)(&_IO_stdout_))    , (unsigned char)(   '\n'   )) : (unsigned char)(*(    ((_IO_FILE*)(&_IO_stdout_))    )->_IO_write_ptr++ = (   '\n'   )))   ;
    }


    md_close(md);

    if( fp != ((_IO_FILE*)(&_IO_stdin_))   )
	fclose(fp);
}



static void
do_test(int times)
{
    MPI base[4];
    MPI exp[4];
    MPI t1 = mpi_alloc(50);
    MPI t2 = mpi_alloc(50);
    MPI t3 = mpi_alloc(50);
    MPI tmp= mpi_alloc(50);
    MPI m =   mpi_alloc(50);
    MPI res = mpi_alloc(50);

    mpi_fromstr( m, "0x10000000000000000000000000" );
    base[0] = mpi_alloc_set_ui( 3 );
    mpi_fromstr( base[0], "0x145984358945989898495ffdd13" );
    base[1] = mpi_alloc_set_ui( 5 );
    mpi_fromstr( base[1], "0x000effff9999000000001100001" );
    base[2] = mpi_alloc_set_ui( 2 );
    mpi_fromstr( base[2], "0x499eeeaaaaa0444444545466672" );
    base[3] = ((void *)0) ;
    exp[0]  = mpi_alloc_set_ui( 30 );
    exp[1]  = mpi_alloc_set_ui( 10 );
    mpi_fromstr( exp[1], "0x3457878888888888aabbbccccc1" );
    exp[2]  = mpi_alloc_set_ui( 24 );
    exp[3] = ((void *)0) ;

    mpi_powm( t1, base[0], exp[0], m );
    mpi_powm( t2, base[1], exp[1], m );
    mpi_powm( t3, base[2], exp[2], m );
    mpi_mulm( tmp, t1, t2, m );
    mpi_mulm( t1, tmp, t3, m );
    log_mpidump("X=", t1 );


    mpi_mulpowm( res, base, exp, m );
    log_mpidump("X=", res );


    m_check(((void *)0) );
}


